@@ -28,6 +28,18 @@
 
 G_BEGIN_DECLS
 
+
+/**
+ * GtkCellRendererState:
+ * @GTK_CELL_RENDERER_SELECTED: The cell is currently selected, and
+ *  probably has a selection colored background to render to.
+ * @GTK_CELL_RENDERER_PRELIT: The mouse is hovering over the cell.
+ * @GTK_CELL_RENDERER_INSENSITIVE: The cell is drawn in an insensitive manner
+ * @GTK_CELL_RENDERER_SORTED: The cell is in a sorted row
+ * @GTK_CELL_RENDERER_FOCUSED: The cell is in the focus row.
+ *
+ * Tells how a cell is to be rendererd.
+ */
 typedef enum
 {
   GTK_CELL_RENDERER_SELECTED    = 1 << 0,
@@ -38,6 +50,17 @@ typedef enum
   GTK_CELL_RENDERER_FOCUSED     = 1 << 4
 } GtkCellRendererState;
 
+/**
+ * GtkCellRendererMode:
+ * @GTK_CELL_RENDERER_MODE_INERT: The cell is just for display
+ *  and cannot be interacted with.  Note that this doesn't mean that eg. the
+ *  row being drawn can't be selected -- just that a particular element of
+ *  it cannot be individually modified.
+ * @GTK_CELL_RENDERER_MODE_ACTIVATABLE: The cell can be clicked.
+ * @GTK_CELL_RENDERER_MODE_EDITABLE: The cell can be edited or otherwise modified.
+ *
+ * Identifies how the user can interact with a particular cell.
+ */
 typedef enum
 {
   GTK_CELL_RENDERER_MODE_INERT,
@@ -88,6 +111,11 @@ struct _GtkCellRendererClass
                                                           gint                  height,
                                                           gint                 *minimum_width,
                                                           gint                 *natural_width);
+  void               (* get_aligned_area)                (GtkCellRenderer      *cell,
+                                                          GtkWidget            *widget,
+							  GtkCellRendererState  flags,
+                                                          const GdkRectangle   *cell_area,
+                                                          GdkRectangle         *aligned_area);
   void               (* get_size)                        (GtkCellRenderer      *cell,
                                                           GtkWidget            *widget,
                                                           const GdkRectangle   *cell_area,
@@ -154,15 +182,20 @@ void               gtk_cell_renderer_get_preferred_size             (GtkCellRend
                                                                      GtkWidget          *widget,
                                                                      GtkRequisition     *minimum_size,
                                                                      GtkRequisition     *natural_size);
-#ifndef GTK_DISABLE_DEPRECATED
+void               gtk_cell_renderer_get_aligned_area               (GtkCellRenderer    *cell,
+								     GtkWidget          *widget,
+								     GtkCellRendererState flags,
+								     const GdkRectangle *cell_area,
+								     GdkRectangle       *aligned_area);
+
+GDK_DEPRECATED_FOR(gtk_cell_renderer_get_preferred_size)
 void             gtk_cell_renderer_get_size       (GtkCellRenderer      *cell,
-						   GtkWidget            *widget,
-						   const GdkRectangle   *cell_area,
-						   gint                 *x_offset,
-						   gint                 *y_offset,
-						   gint                 *width,
-						   gint                 *height);
-#endif
+                                                   GtkWidget            *widget,
+                                                   const GdkRectangle   *cell_area,
+                                                   gint                 *x_offset,
+                                                   gint                 *y_offset,
+                                                   gint                 *width,
+                                                   gint                 *height);
 void             gtk_cell_renderer_render         (GtkCellRenderer      *cell,
                                                    cairo_t              *cr,
 						   GtkWidget            *widget,
@@ -213,18 +246,24 @@ void             gtk_cell_renderer_set_sensitive  (GtkCellRenderer      *cell,
                                                    gboolean              sensitive);
 gboolean         gtk_cell_renderer_get_sensitive  (GtkCellRenderer      *cell);
 
+gboolean         gtk_cell_renderer_is_activatable (GtkCellRenderer      *cell);
+
 /* For use by cell renderer implementations only */
 void             gtk_cell_renderer_stop_editing   (GtkCellRenderer      *cell,
                                                    gboolean              canceled);
 
 
 void            _gtk_cell_renderer_calc_offset    (GtkCellRenderer      *cell,
-						   const GdkRectangle   *cell_area,
-						   GtkTextDirection      direction,
-						   gint                  width,
-						   gint                  height,
-						   gint                 *x_offset,
-						   gint                 *y_offset);
+                                                   const GdkRectangle   *cell_area,
+                                                   GtkTextDirection      direction,
+                                                   gint                  width,
+                                                   gint                  height,
+                                                   gint                 *x_offset,
+                                                   gint                 *y_offset);
+
+GtkStateFlags   gtk_cell_renderer_get_state       (GtkCellRenderer      *cell,
+                                                   GtkWidget            *widget,
+                                                   GtkCellRendererState  cell_state);
 
 G_END_DECLS
 
