@@ -21,11 +21,34 @@
  */
 
 #include "config.h"
-#include <gdk/gdkkeysyms.h>
+
+#include "gtkcellrendererspin.h"
+
 #include "gtkintl.h"
 #include "gtkprivate.h"
 #include "gtkspinbutton.h"
-#include "gtkcellrendererspin.h"
+
+
+/**
+ * SECTION:gtkcellrendererspin
+ * @Short_description: Renders a spin button in a cell
+ * @Title: GtkCellRendererSpin
+ * @See_also: #GtkCellRendererText, #GtkSpinButton
+ *
+ * #GtkCellRendererSpin renders text in a cell like #GtkCellRendererText from
+ * which it is derived. But while #GtkCellRendererText offers a simple entry to
+ * edit the text, #GtkCellRendererSpin offers a #GtkSpinButton widget. Of course,
+ * that means that the text has to be parseable as a floating point number.
+ *
+ * The range of the spinbutton is taken from the adjustment property of the
+ * cell renderer, which can be set explicitly or mapped to a column in the
+ * tree model, like all properties of cell renders. #GtkCellRendererSpin
+ * also has properties for the #GtkCellRendererSpin:climb-rate and the number
+ * of #GtkCellRendererSpin:digits to display. Other #GtkSpinButton properties
+ * can be set in a handler for the #GtkCellRenderer::editing-started signal.
+ *
+ * The #GtkCellRendererSpin cell renderer was added in GTK+ 2.10.
+ */
 
 
 struct _GtkCellRendererSpinPrivate
